/*
 * $Id: Ud.java,v 1.2 2003/06/14 13:56:33 kenta Exp $
 *
 * Copyright (c) 2003 Kenta Cho. All rights reserved.
 */
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Random;

import javax.microedition.io.Connector;

import com.nttdocomo.lang.IllegalStateException;
import com.nttdocomo.opt.ui.j3d2.C3DGraphics;
import com.nttdocomo.opt.ui.j3d2.C3DMath;
import com.nttdocomo.system.InterruptedOperationException;
import com.nttdocomo.ui.Display;
import com.nttdocomo.ui.Frame;
import com.nttdocomo.ui.MApplication;
import com.nttdocomo.ui.UIException;

/**
 * UrbanDrivin main class.
 */
public class Ud extends MApplication {
	private SimpleC3DCanvas canvas;
	private C3DGraphics g;
	private int scWidth, scHeight;

	private final int fx1 = 65536;

	/**
	 * Initialize.
	 */
	private void init() {
		canvas = new SimpleC3DCanvas();
		scWidth = canvas.getWidth();
		scHeight = canvas.getHeight();
		Display.setCurrent(canvas);
	}

	/**
	 * Init a canvas after the C3DGraphics instance becomes available.
	 */
	private void initCanvas()
		throws UIException, InterruptedOperationException, IOException {

		g.set3DMode(true);
		g.viewport(0, 0, scWidth, scHeight);

		g.enable(C3DGraphics.C3DL_TEXTURE_2D);
		initStringTextures();
		g.disable(C3DGraphics.C3DL_TEXTURE_2D);

		g.blendFunc(
			C3DGraphics.C3DL_SRC_ALPHA,
			C3DGraphics.C3DL_ONE_MINUS_SRC_ALPHA);
		g.enable(C3DGraphics.C3DL_BLEND);
		g.enable(C3DGraphics.C3DL_CULL_FACE);
		g.disable(C3DGraphics.C3DL_DITHER);

		g.matrixMode(C3DGraphics.C3DL_PROJECTION);
		g.loadIdentity();
		int aspect = fx1 * scWidth / scHeight;
		g.perspective(45 * fx1, aspect, fx1 / 256, 512 * fx1);

		g.matrixMode(C3DGraphics.C3DL_MODELVIEW);
		g.loadIdentity();

		initGamestate();
	}

	private void gClearColor(int cr, int cg, int cb, int a) {
		switch (ctz) {
			case 0 :
				cr += cr >> 2;
				cg += cg >> 2;
				cb += cb >> 3;
				if (cr > fx1)
					cr = fx1;
				if (cg > fx1)
					cg = fx1;
				if (cb > fx1)
					cb = fx1;
				break;
			case 2 :
				cr += cr >> 2;
				cg -= cr >> 2;
				cb -= cb >> 1;
				if (cr > fx1)
					cr = fx1;
				break;
			case 3 :
				cr -= cr >> 1;
				cg -= cg >> 1;
				break;
			case 4 :
				cr >>= 1;
				cg >>= 1;
				cb >>= 1;
				break;
		}
		g.clearColor(cr, cg, cb, a);
	}

	private void gColor(int cr, int cg, int cb, int a) {
		switch (ctz) {
			case 0 :
				cr += cr >> 2;
				cg += cg >> 2;
				cb += cb >> 3;
				if (cr > fx1)
					cr = fx1;
				if (cg > fx1)
					cg = fx1;
				if (cb > fx1)
					cb = fx1;
				break;
			case 2 :
				cr += cr >> 2;
				cg -= cr >> 2;
				cb -= cb >> 1;
				if (cr > fx1)
					cr = fx1;
				break;
			case 3 :
				cr -= cr >> 1;
				cg -= cg >> 1;
				break;
			case 4 :
				cr >>= 1;
				cg >>= 1;
				cb >>= 1;
				break;
		}
		g.color(cr, cg, cb, a);
	}

	/*
	 * Make the course and the display list.
	 */
	// Curve.
	private int[][] csCrv = new int[5][80];
	// Up down.
	private int[][] csUd = new int[5][80];
	// Position.
	private int[][][][] csPos = new int[5][80][3][2];
	// Position of an end of the cource(to connect the next display list).
	private int[][] csEpos = new int[5][3];
	private int[][] csEdeg = new int[5][2];
	// Ground color.
	private int[][][] csClr = new int[5][4][2];
	// Course section length.
	private int csl;
	// Course section num.
	private int csn;

	private int sox, soy, soz;

	private void calcSideObjectPos(int ci, int cn, int o) {
		int x1 = csPos[ci][cn][0][0], x2 = csPos[ci][cn][0][1];
		soy = csPos[ci][cn][1][0];
		int z1 = csPos[ci][cn][2][0], z2 = csPos[ci][cn][2][1];
		int ox = x2 - x1, oz = z2 - z1;
		int cx = (x1 + x2) / 2, cz = (z1 + z2) / 2;
		sox = cx + ((ox * o) >> 8);
		soz = cz + ((oz * o) >> 8);
	}

	/**
	 * Add a building model.
	 * @param ci
	 * @param cn
	 * @param xo
	 * @param xos
	 * @param tp  Type. 0- Normal. 1- Tall. 2- Small. 3- Forest 4- Mountain.
	 */
	private void addBuilding(int ci, int cn, int xo, int xos, int tp) {
		int bw, bd, bh;
		int bx1, bx2, bx3, bx4;
		int bz1, bz2, bz3, bz4;
		int by1, by3;
		switch (tp) {
			default :
				bw = randN(512) + 256;
				bd = randN(3) + 1;
				bh = randN(fx1 * 5) + fx1;
				break;
			case 1 :
				bw = randN(300) + 200;
				bd = randN(3) + 1;
				bh = randN(fx1 * 18) + fx1 * 15;
				xo = xo * 2;
				break;
			case 2 :
				bw = randN(200) + 200;
				bd = randN(2) + 1;
				bh = randN(fx1 * 3) + fx1;
				break;
			case 3 :
				bw = randN(200) + 400;
				bd = randN(2) + 2;
				bh = randN(fx1 * 5) + fx1 * 3;
				break;
			case 4 :
				bw = randN(200) + 400;
				bd = 3;
				bh = randN(fx1 * 20) + fx1 * 8;
				break;
		}
		if (cn + bd >= csl)
			return;

		calcSideObjectPos(ci, cn, xo * xos);
		bx1 = sox;
		by1 = soy;
		bz1 = soz;
		calcSideObjectPos(ci, cn, (xo + bw) * xos);
		bx2 = sox;
		bz2 = soz;
		calcSideObjectPos(ci, cn + bd, xo * xos);
		bx3 = sox;
		by3 = soy;
		bz3 = soz;
		calcSideObjectPos(ci, cn + bd, (xo + bw) * xos);
		bx4 = sox;
		bz4 = soz;
		if (xos == -1) {
			int tx = bx1, tz = bz1;
			bx1 = bx2;
			bz1 = bz2;
			bx2 = tx;
			bz2 = tz;
			tx = bx3;
			tz = bz3;
			bx3 = bx4;
			bz3 = bz4;
			bx4 = tx;
			bz4 = tz;
		}
		int br, bg, bb;
		switch (tp) {
			default :
				br = randN(fx1 / 2) + fx1 / 2;
				bg = randN(fx1 / 2) + fx1 / 2;
				bb = randN(fx1 / 2) + fx1 / 2;
				break;
			case 3 :
				br = randN(fx1 / 8) + fx1 / 4;
				bg = fx1 / 6 * 5;
				bb = randN(fx1 / 8) + fx1 / 4;
				break;
			case 4 :
				br = randN(fx1 / 4) + fx1 / 4 * 3;
				bg = randN(fx1 / 4) + fx1 / 2;
				bb = randN(fx1 / 8) + fx1 / 4;
				break;
		}
		gColor(br / 2, bg / 2, bb / 2, fx1 / 8 * 7);
		g.vertex(bx1, by1, bz1);
		g.vertex(bx2, by1, bz2);
		gColor(br, bg, bb, fx1 / 6 * 5);
		g.vertex(bx2, by1 + bh, bz2);
		g.vertex(bx1, by1 + bh, bz1);
		gColor(br / 2, bg / 2, bb / 2, fx1 / 8 * 7);
		g.vertex(bx3, by3, bz3);
		g.vertex(bx1, by1, bz1);
		gColor(br, bg, bb, fx1 / 6 * 5);
		g.vertex(bx1, by1 + bh, bz1);
		g.vertex(bx3, by3 + bh, bz3);
		gColor(br / 2, bg / 2, bb / 2, fx1 / 8 * 7);
		g.vertex(bx2, by1, bz2);
		g.vertex(bx4, by3, bz4);
		gColor(br, bg, bb, fx1 / 8 * 7);
		g.vertex(bx4, by3 + bh, bz4);
		g.vertex(bx2, by1 + bh, bz2);
	}

	private void addBar(
		int ci,
		int cn,
		int xo,
		int xos,
		int w,
		int h,
		int br,
		int bg,
		int bb) {
		int bx1, bx2;
		int bz1, bz2;
		int by1;
		calcSideObjectPos(ci, cn, xo * xos);
		bx1 = sox;
		by1 = soy;
		bz1 = soz;
		calcSideObjectPos(ci, cn, (xo + w) * xos);
		bx2 = sox;
		bz2 = soz;
		if (xos == -1) {
			int tx = bx1, tz = bz1;
			bx1 = bx2;
			bz1 = bz2;
			bx2 = tx;
			bz2 = tz;
		}
		gColor(br / 2, bg / 2, bb / 2, fx1 / 6 * 5);
		g.vertex(bx1, by1, bz1);
		g.vertex(bx2, by1, bz2);
		gColor(br, bg, bb, fx1 / 6 * 5);
		g.vertex(bx2, by1 + h, bz2);
		g.vertex(bx1, by1 + h, bz1);
	}

	private void addOverBar(
		int ci,
		int cn,
		int xo,
		int yo,
		int h,
		int br,
		int bg,
		int bb) {
		int bx1, bx2;
		int bz1, bz2;
		int by1;
		calcSideObjectPos(ci, cn, -xo);
		bx1 = sox;
		by1 = soy + yo;
		bz1 = soz;
		calcSideObjectPos(ci, cn, xo);
		bx2 = sox;
		bz2 = soz;
		gColor(br / 2, bg / 2, bb / 2, fx1 / 6 * 5);
		g.vertex(bx1, by1, bz1);
		g.vertex(bx2, by1, bz2);
		gColor(br, bg, bb, fx1 / 6 * 5);
		g.vertex(bx2, by1 + h, bz2);
		g.vertex(bx1, by1 + h, bz1);
	}

	private void addGate(int ci, int cn) {
		int gx1, gy1, gz1;
		int gx2, gy2, gz2;
		int gx3, gy3, gz3;
		int gx4, gy4, gz4;
		calcSideObjectPos(ci, cn, -512);
		gx1 = sox;
		gy1 = soy - fx1 * 3;
		gz1 = soz;
		calcSideObjectPos(ci, cn, -256);
		gx2 = sox;
		gy2 = soy - fx1 * 3;
		gz2 = soz;
		calcSideObjectPos(ci, cn, 0);
		gx3 = sox;
		gy3 = soy + fx1 * 20;
		gz3 = soz;
		gx4 = sox;
		gy4 = soy + fx1 * 34;
		gz4 = soz;
		gColor(fx1 / 4 * 3, fx1 / 4 * 3, fx1 / 4 * 3, fx1 / 4);
		g.vertex(gx1, gy1, gz1);
		g.vertex(gx2, gy2, gz2);
		gColor(fx1 / 4 * 3, fx1 / 4 * 3, fx1 / 4 * 3, fx1);
		g.vertex(gx3, gy3, gz3);
		g.vertex(gx4, gy4, gz4);
		gColor(fx1 / 4 * 3, fx1 / 4 * 3, fx1 / 4 * 3, fx1 / 4);
		g.vertex(-gx2, gy1, gz1);
		g.vertex(-gx1, gy2, gz2);
		gColor(fx1 / 4 * 3, fx1 / 4 * 3, fx1 / 4 * 3, fx1);
		g.vertex(gx4, gy4, gz4);
		g.vertex(gx3, gy3, gz3);
	}

	// Course Type.
	// 0- City. 1- Town. 2- Suburb. 3- Seaside. 4- Lakeside. 5- Mountain 6- Forest.
	// Curve level.
	private final int[] cl = { 8, 12, 6, 9, 10, 15, 12 };
	// Curve frequency.
	private final int[] cf = { 15, 10, 12, 20, 11, 8, 10 };
	// Updown level.
	private final int[] ul = { 2, 5, 10, 1, 14, 18, 6 };
	// Updown frequency.
	private final int[] uf = { 16, 12, 8, 9, 10, 20, 22 };
	// Ground color.
	private final int[][][] gc =
		{ { { fx1 / 8 * 5, fx1 / 8 * 5, fx1 / 8 * 6, fx1 }, {
				fx1 / 8 * 5, fx1 / 8 * 5, fx1 / 8 * 6, fx1 }
		}, {
			{
				fx1 / 8 * 5, fx1 / 8 * 6, fx1 / 8 * 4, fx1 }, {
				fx1 / 8 * 5, fx1 / 8 * 6, fx1 / 8 * 4, fx1 }
		}, {
			{
				fx1 / 8 * 4, fx1 / 8 * 6, fx1 / 8 * 3, fx1 }, {
				fx1 / 8 * 4, fx1 / 8 * 6, fx1 / 8 * 3, fx1 }
		}, {
			{
				fx1 / 8 * 7, fx1 / 8 * 7, fx1 / 8 * 4, fx1 }, {
				fx1 / 8 * 4, fx1 / 8 * 4, fx1 / 8 * 8, fx1 / 8 * 7 }
		}, {
			{
				fx1 / 8 * 4, fx1 / 8 * 7, fx1 / 8 * 3, fx1 }, {
				fx1 / 8 * 3, fx1 / 8 * 3, fx1 / 8 * 8, fx1 / 8 * 7 }
		}, {
			{
				fx1 / 8 * 8, fx1 / 8 * 5, fx1 / 8 * 5, fx1 }, {
				fx1 / 8 * 8, fx1 / 8 * 8, fx1 / 8 * 8, fx1 / 8 }
		}, {
			{
				fx1 / 8 * 3, fx1 / 8 * 6, fx1 / 8 * 3, fx1 }, {
				fx1 / 8 * 3, fx1 / 8 * 6, fx1 / 8 * 3, fx1 }
		}, };
	private boolean[] sh = { false, false, false, true, true, true, false };

	// Ground color.
	private int[][] gcl = new int[4][2];
	private int[][] gacl = new int[4][2];

	// Side of the sea, river and moutain.
	private int sd, rsd;

	// Current course type;
	private int cct;

	private int c, u;

	/**
	 * Make a course data and objects around the road.
	 * 
	 * @param n  Course number.
	 * @param tp Course type.
	 * @param rk Game rank(16: normal).
	 */
	private void makeCourse(int n, int tp, int rk) {
		int i;
		int cd, ac = 0, cc = 0, rc;
		int ud, au = 0;

		int cln = cl[tp] * rk;
		int cfn = cf[tp];
		int uln = ul[tp] * 16;
		int ufn = uf[tp];

		for (i = 0; i < 4; i++) {
			csClr[n][i][sd] = gc[tp][0][i];
			csClr[n][i][rsd] = gc[tp][1][i];
		}

		int sd1, sd2;
		switch (tp) {
			default :
				sd1 = sd2 = 16;
				break;
			case 3 :
			case 4 :
			case 5 :
				if (sd == 0) {
					sd1 = 32;
					sd2 = 1;
				} else {
					sd1 = 1;
					sd2 = 32;
				}
				break;
		}

		cd = randN(cfn / 2) + cfn / 2;
		ud = randN(ufn / 2) + ufn / 2;
		for (i = 0; i < csl; i++) {
			cd--;
			rc = (cc >> 4);
			if (cd <= 0 || rc > 200 || rc < -200) {
				if (rc > 200) {
					ac = -randN(cln);
				} else if (rc < -200) {
					ac = randN(cln);
				} else {
					ac = randNS(cln);
				}
				cd = randN(cfn) + cfn / 2;
			}
			ud--;
			if (ud <= 0) {
				au = randNS(uln);
				ud = randN(ufn) + ufn / 2;
			}
			if (i > csl - 10) {
				ac = au = 0;
				cd = ud = 99999;
			}
			c += (ac - c) >> 2;
			cc += c;
			csCrv[n][i] = c;
			u += (au - u) >> 2;
			csUd[n][i] = u;
		}

		int x = 0, y = 0, z = 0;
		int x1 = fx1, z1 = 0, x2 = -fx1, z2 = 0;
		int px1, pz1, px2, pz2, py;
		int cl;
		cc = 0;
		for (i = 0; i < csl; i++) {
			csPos[n][i][0][0] = x1;
			csPos[n][i][1][0] = y;
			csPos[n][i][2][0] = z1;
			csPos[n][i][0][1] = x2;
			csPos[n][i][1][1] = y;
			csPos[n][i][2][1] = z2;
			px1 = x1;
			pz1 = z1;
			px2 = x2;
			pz2 = z2;
			py = y;
			cc += csCrv[n][i];
			rc = ((cc >> 4) & 1023);
			x += sctbl[rc] << 2;
			z += sctbl[rc + 256] << 2;
			y += csUd[n][i] << 8;
			x1 = x + sctbl[rc + 256];
			z1 = z - sctbl[rc];
			x2 = x + x - x1;
			z2 = z + z - z1;
		}
		csEpos[n][0] = (csPos[n][csl - 1][0][0] + csPos[n][csl - 1][0][1]) / 2;
		csEpos[n][1] = csPos[n][csl - 1][1][0];
		csEpos[n][2] = (csPos[n][csl - 1][2][0] + csPos[n][csl - 1][2][1]) / 2;
		csEdeg[n][0] =
			C3DMath.atan2(
				csEpos[n][0]
					- (csPos[n][csl - 2][0][0] + csPos[n][csl - 2][0][1]) / 2,
				csEpos[n][2]
					- (csPos[n][csl - 2][2][0] + csPos[n][csl - 2][2][1]) / 2);
		csEdeg[n][1] =
			C3DMath.atan2(
				csEpos[n][1]
					- (csPos[n][csl - 2][1][0] + csPos[n][csl - 2][1][1]) / 2,
				(fx1 << 2));
		g.newList(n + 1, C3DGraphics.C3DL_COMPILE);
		g.begin(C3DGraphics.C3DL_QUADS);
		for (i = csl - 1; i >= 0; i--) {
			px1 = x1;
			pz1 = z1;
			px2 = x2;
			pz2 = z2;
			py = y;
			x1 = csPos[n][i][0][0];
			y = csPos[n][i][1][0];
			z1 = csPos[n][i][2][0];
			x2 = csPos[n][i][0][1];
			z2 = csPos[n][i][2][1];
			if (i == csl - 1)
				continue;
			cl = randN(fx1 / 4) + fx1 / 4 * 2;
			gColor(cl, cl, cl, fx1 / 10 * 9);
			g.vertex(px2, py, pz2);
			g.vertex(px1, py, pz1);
			cl = randN(fx1 / 4) + fx1 / 4 * 3;
			gColor(cl, cl, cl, fx1 / 10 * 9);
			g.vertex(x1, y, z1);
			g.vertex(x2, y, z2);
			if ((i & 3) == 0) {
				x = (x1 + x2) / 2;
				z = (z1 + z2) / 2;
				gColor(
					csClr[n][0][sd],
					csClr[n][1][sd],
					csClr[n][2][sd],
					fx1 / 8 * 7);
				g.vertex(
					x + ((x2 - x1) * sd1 / 2),
					y,
					z + ((z2 - z1) * sd1 / 2));
				g.vertex(
					x - ((x2 - x1) * sd2 / 2),
					y,
					z - ((z2 - z1) * sd2 / 2));
				gColor(
					csClr[n][0][sd] / 3 * 2,
					csClr[n][1][sd] / 3 * 2,
					csClr[n][2][sd] / 3 * 2,
					fx1 / 8 * 7);
				g.vertex(
					x - ((x2 - x1) * sd2 / 2),
					y - fx1 * 4,
					z - ((z2 - z1) * sd2 / 2));
				g.vertex(
					x + ((x2 - x1) * sd1 / 2),
					y - fx1 * 4,
					z + ((z2 - z1) * sd1 / 2));
			}
			if (n == 0 && i == 8) {
				addGate(n, i);
			}
			switch (tp) {
				case 0 :
					if ((i & 3) == 0) {
						if (sd == 0 && randN(24) == 0) {
							addBuilding(n, i, 160, -1, 1);
						} else {
							addBuilding(n, i, 160, -1, 0);
						}
						if (sd == 1 && randN(24) == 0) {
							addBuilding(n, i, 160, 1, 1);
						} else {
							addBuilding(n, i, 160, 1, 0);
						}
						if (randN(5) == 0) {
							addOverBar(
								n,
								i,
								1024,
								fx1 * 2,
								fx1 / 2,
								fx1 / 2,
								fx1 / 2,
								fx1 / 2);
						}
					}
					break;
				case 1 :
					if ((i & 3) == 0) {
						if (randN(4) < 3) {
							addBuilding(n, i, randN(2048) + 128, 1, 2);
						}
						if (randN(4) < 3) {
							addBuilding(n, i, randN(2048) + 128, -1, 2);
						}
					}
					if ((i & 7) == 2) {
						addBar(
							n,
							i,
							130,
							1,
							16,
							fx1 * 2,
							fx1 / 2,
							fx1 / 3,
							fx1 / 3);
					} else if ((i & 7) == 6) {
						addBar(
							n,
							i,
							130,
							-1,
							16,
							fx1 * 2,
							fx1 / 2,
							fx1 / 3,
							fx1 / 3);
					}
					break;
				case 2 :
					if (randN(8) == 0) {
						addBuilding(
							n,
							i,
							randN(4096) + 1024,
							randN(2) * 2 - 1,
							2);
					} else if (randN(2) == 0) {
						addBar(
							n,
							i,
							randN(512) + 128,
							randN(2) * 2 - 1,
							64 + randN(128),
							randN(fx1 * 2) + fx1 / 4,
							fx1 / 4 + randN(fx1 / 4),
							fx1 / 2 + randN(fx1 / 2),
							fx1 / 5 + randN(fx1 / 5));
					}
					break;
				case 3 :
					if ((i & 7) == 0) {
						addBuilding(n, i, randN(256) + 512, rsd * 2 - 1, 2);
					}
					if ((i & 3) == 0) {
						addBar(
							n,
							i,
							randN(128) + 128,
							rsd * 2 - 1,
							128 + randN(64),
							randN(fx1 / 2) + fx1 / 2,
							fx1 / 4 + randN(fx1 / 4),
							fx1 / 2 + randN(fx1 / 2),
							fx1 / 5 + randN(fx1 / 5));
					}
					break;
				case 4 :
					if ((i & 3) == 0) {
						addBuilding(n, i, randN(2048) + 256, rsd * 2 - 1, 3);
					}
					break;
				case 5 :
					if ((i & 3) == 0) {
						addBuilding(n, i, randN(1024) + 128, rsd * 2 - 1, 4);
					}
					break;
				case 6 :
					if ((i & 3) == 0) {
						addBuilding(n, i, randN(256) + 400, 1, 3);
						addBuilding(n, i, randN(256) + 400, -1, 3);
					}
					break;
			}
		}
		g.end();
		g.endList();
	}

	// Course type.
	private int ctt;
	private int[][] ct = { { 4, 0, 0, 1, 2 }, // City.
		{
			5, 3, 1, 0, 3, 2 }, // Seaside.
		{
			3, 5, 5, 5 }, // Mountain.
		{
			4, 6, 6, 2, 6 }, // Forest.
		{
			5, 4, 6, 2, 6, 4 }, // Lakeside.
		{
			5, 2, 1, 2, 0, 1 }, // Suburb.
		{
			2, 4, 4 }, // Demo.
	};
	// Course timezone. 0- Morning 1- Noon 2- Afternoon 3- Night 4- Demo.
	private int ctz;
	// Difficulty. 0- Easy 1- Normal 2- Hard 3- Random.
	private int dfc;

	private void initStage(int t, int d, int s) {
		ctt = t;
		csn = ct[ctt][0];
		dfc = d;

		int dp;
		if (d == 3) {
			dp = 12 + randN(30);
			rnd.setSeed(sdRnd.nextLong());
		} else {
			dp = 16 + dfc * 10;
			rnd.setSeed(s);
		}
		sd = randN(2);
		rsd = (sd - 1) & 1;
		csl = 64 + randN(16); // Length of each course section(<=80).
		if (t == 6) {
			ctz = 4;
		} else {
			ctz = randN(4);
		}
		gClearColor(fx1 / 2, fx1 / 3 * 2, fx1 / 4 * 3, fx1);
		c = u = 0;
		for (int i = 0; i < csn; i++) {
			makeCourse(i, ct[ctt][i + 1], dp);
		}
		for (int i = 0; i < 4; i++) {
			gcl[i][0] = gacl[i][0] = csClr[0][i][0];
			gcl[i][1] = gacl[i][1] = csClr[0][i][1];
		}
		cct = ct[ctt][1];
	}

	// Game state 0- Title 1- Start 2- InGame 3- Gameover 4- Clear 5- Pause.
	private int state;
	// Left time.
	private int ltm;
	// Increases letf time per lap. 
	private int itp;
	// Course time.
	private int ctm;
	// Lap.
	private int lap;
	// Lap display cnt.
	private int ldc;
	// Total lap.
	private int tlp;
	// Selected course type.
	private int sct;
	// Selected difficulty.
	private int sdf;

	// Records.
	private int[][] rcd = new int[6][4];

	private void loadRecords() throws IOException {
		InputStream in = Connector.openInputStream("scratchpad:///0");
		try {
			if (in.read() != 10) {
				throw new IOException("Invalid version num");
			}
			for (int i = 0; i < 6; i++) {
				for (int j = 0; j < 4; j++) {
					rcd[i][j] =
						(in.read() << 24)
							| (in.read() << 16)
							| (in.read() << 8)
							| in.read();
				}
			}
			sct = in.read();
			sdf = in.read();
		} finally {
			in.close();
		}
	}

	private void saveReocrds() throws IOException {
		OutputStream out = Connector.openOutputStream("scratchpad:///0");
		try {
			out.write(10);
			for (int i = 0; i < 6; i++) {
				for (int j = 0; j < 4; j++) {
					int r = rcd[i][j];
					out.write((r >> 24) & 0xff);
					out.write((r >> 16) & 0xff);
					out.write((r >> 8) & 0xff);
					out.write(r & 0xff);
				}
			}
			out.write(sct);
			out.write(sdf);
		} finally {
			out.close();
		}
	}

	private void initRecords() {
		for (int i = 0; i < 6; i++) {
			for (int j = 0; j < 4; j++) {
				rcd[i][j] = (9 * 60 * 100 + 59 * 100 + 99) * 10;
			}
		}
	}

	private void initGamestate() {
		try {
			loadRecords();
		} catch (Exception e) {
			initRecords();
			sct = sdf = 0;
		}
		initTitle();
	}

	private void initTitle() {
		canvas.setSoftLabel(Frame.SOFT_KEY_1, "Jn");
		canvas.setSoftLabel(Frame.SOFT_KEY_2, "I");
		initCar();
		ct[6][1] = ct[6][2] = randN(7);
		initStage(6, 3, 0);
		state = 0;
	}

	// Course data random seeds.
	static int[][] crs = { { 11324, 56105, 26418, -1 }, {
			61166, 22222, 44245, -1 }, {
			27870, 1, 1111, -1 }, {
			1, 33861, 55232, -1 }, {
			62345, 123, 3333, -1 }, {
			1, 12486, 43999, -1 }, };

	private void initGame() {
		canvas.setSoftLabel(Frame.SOFT_KEY_1, "f");
		canvas.setSoftLabel(Frame.SOFT_KEY_2, "");
		initCar();
		initStage(sct, sdf, crs[sct][sdf]);
		tlp = 1 + dfc;
		ltm = (csl * ct[sct][0] * 3 / 20) * 20 + 19;
		itp = (csl * ct[sct][0] * 100 / 864/*5 / 6 / 20*/) * 20;
		if (dfc == 3)
			tlp = 3;
		ctm = 0;
		lap = 0;
		ldc = Integer.MAX_VALUE;
		cnt = 0;
		state = 1;
	}

	private void initGameover() {
		canvas.setSoftLabel(Frame.SOFT_KEY_1, "");
		try {
			saveReocrds();
		} catch (Exception e) {}
		cnt = 0;
		state = 3;
	}

	private void initClear() {
		canvas.setSoftLabel(Frame.SOFT_KEY_1, "");
		if (rcd[sct][sdf] > ctm) {
			rcd[sct][sdf] = ctm;
		}
		try {
			saveReocrds();
		} catch (Exception e) {}
		cnt = 0;
		state = 4;
	}

	private void initPause() {
		canvas.setSoftLabel(Frame.SOFT_KEY_1, "ĊJ");
		state = 5;
	}

	private void resumePause() {
		canvas.setSoftLabel(Frame.SOFT_KEY_1, "f");
		state = 2;
	}

	// Viewpoint data.
	private int ex, ey, ez, eax, eay, eaz;
	private int sx, sy, sz, sax, say, saz;

	/**
	 * Move an eye position and a looking position.
	 */
	private void moveViewpoint() {
		ex += (eax - ex) >> 2;
		ey += (eay - ey) >> 2;
		ez += (eaz - ez) >> 2;
	}

	/**
	 * Adjust a viewpoint when the car moves to the next display list.
	 */
	private void adjustViewpoint(int mx, int my, int mz, int d1, int d2) {
		int ox, oy, oz;
		ox = ex - mx;
		oz = ez - mz;
		ex =
			(int) (((long) ox * sctbl[d1 + 256] - (long) oz * sctbl[d1]) >> 16);
		ez =
			(int) (((long) ox * sctbl[d1] + (long) oz * sctbl[d1 + 256]) >> 16);
		oy = ey - my;
		oz = ez;
		ey =
			(int) (((long) oy * sctbl[d2 + 256] - (long) oz * sctbl[d2]) >> 16);
		ez =
			(int) (((long) oy * sctbl[d2] + (long) oz * sctbl[d2 + 256]) >> 16);
		ox = eax - mx;
		oz = eaz - mz;
	}

	/*
	 * Handle my car.
	 */
	// Potision.
	private int cci, ci, co, cw, cs, ncci;
	// Movement.
	private int cmw, cd3;
	// Position in the OpenGL world.
	private int cx, cy, cz;
	// Out of the course.
	private boolean coc;
	// Break.
	private boolean brk;

	private void initCar() {
		cci = ci = co = cs = 0;
		ncci = 1;
		cw = fx1 / 2;
		cx = cy = cz = 0;
		cmw = cd3 = 0;
		eax = eay = eaz = 0;
		ex = ey = ez = 0;
		coc = false;
	}

	private void moveCar(int key) {
		int x1, y1, z1, x2, y2, z2;
		int acmw;
		switch (state) {
			case 1 :
			case 2 :
				// Break control.
				if ((key
					& ((1 << Display.KEY_DOWN)
						| (1 << Display.KEY_LOWER_LEFT)
						| (1 << Display.KEY_LOWER_RIGHT)))
					!= 0) {
					cs -= 512;
					brk = true;
				} else {
					if (!coc) {
						if (cs < fx1 / 2) {
							cs += (fx1 / 2 - cs) >> 6;
						} else if (cs < fx1 - (fx1 / 1024)) {
							cs += (fx1 / 1024);
						}
					}
					brk = false;
				}
				if (coc) {
					cs -= cs >> 5;
				}
				// Steering control.
				if ((key
					& ((1 << Display.KEY_LEFT)
						| (1 << Display.KEY_LOWER_LEFT)
						| (1 << Display.KEY_UPPER_LEFT)))
					!= 0) {
					if (brk)
						acmw = -fx1 / 20;
					else
						acmw = -fx1 / 32;
					cs -= 64;
				} else if (
					(key
						& ((1 << Display.KEY_RIGHT)
							| (1 << Display.KEY_LOWER_RIGHT)
							| (1 << Display.KEY_UPPER_RIGHT)))
						!= 0) {
					if (brk)
						acmw = fx1 / 20;
					else
						acmw = fx1 / 32;
					cs -= 64;
				} else {
					acmw = 0;
				}
				cmw += (acmw - cmw) >> 4;
				cw += cmw;
				cw += (csCrv[cci][ci] * cs) >> 12;
				if (cw <= 0) {
					if (sh[cct] && sd == 0) {
						cw = fx1 / 8;
						cs >>= 1;
						if (cmw < 0)
							cmw = -cmw;
					} else {
						coc = true;
						if (cw <= -fx1 / 6) {
							cw = -fx1 / 6;
							if (cmw < 0)
								cmw = -cmw;
						}
					}
				} else if (cw >= fx1) {
					if (sh[cct] && sd == 1) {
						cw = fx1 / 8 * 7;
						cs >>= 1;
						if (cmw > 0)
							cmw = -cmw;
					} else {
						coc = true;
						if (cw >= fx1 / 6 * 7) {
							cw = fx1 / 6 * 7;
							if (cmw > 0)
								cmw = -cmw;
						}
					}
				} else {
					coc = false;
				}
				break;
			case 0 :
			case 3 :
			case 4 :
				coc = false;
				cw += (fx1 / 2 - cw) >> 3;
				switch (state) {
					case 0 :
						cs += (fx1 / 4 - cs) >> 3;
						break;
					case 3 :
						cs -= cs >> 3;
						break;
					case 4 :
						cs += (fx1 / 2 - cs) >> 3;
						break;
				}
				cmw = 0;
				break;
		}

		// Move the car position.
		cd3 += ((-cmw >> 5) - cd3) >> 1;
		if (cs < 0)
			cs = 0;
		co += cs;
		if (co >= fx1) {
			co -= fx1;
			ci++;
			if (cci == 0 && ci == 8 && state != 0 && state != 4) {
				// Go to the next lap.
				state = 2;
				if (state == 2) {
					lap++;
					if (lap > tlp) {
						initClear();
					} else {
						ldc = cnt;
						if (lap > 1) {
							int ri = itp - lap * 40;
							if (ri < 200)
								ri = 200;
							ltm += ri;
						}
					}
				}
			}
			if (ci >= csl - 1) {
				// Go to the next course section.
				ci = 0;
				adjustViewpoint(
					csEpos[cci][0],
					csEpos[cci][1],
					csEpos[cci][2],
					((csEdeg[cci][0] / 360 * 1024 / fx1) & 1023),
					((csEdeg[cci][1] / 360 * 1024 / fx1) & 1023));
				cci++;
				ncci++;
				if (cci >= csn)
					cci = 0;
				if (ncci >= csn)
					ncci = 0;
				for (int i = 0; i < 4; i++) {
					gacl[i][0] = csClr[cci][i][0];
					gacl[i][1] = csClr[cci][i][1];
				}
				cct = ct[ctt][cci + 1];
			}
		}
		x1 =
			(int) (((long) csPos[cci][ci][0][0] * (fx1 - cw)
				+ (long) csPos[cci][ci][0][1] * cw)
				>> 16);
		y1 = csPos[cci][ci][1][0];
		z1 =
			(int) (((long) csPos[cci][ci][2][0] * (fx1 - cw)
				+ (long) csPos[cci][ci][2][1] * cw)
				>> 16);
		x2 =
			(int) (((long) csPos[cci][ci
				+ 1][0][0] * (fx1 - cw)
				+ (long) csPos[cci][ci
				+ 1][0][1] * cw)
				>> 16);
		y2 = csPos[cci][ci + 1][1][0];
		z2 =
			(int) (((long) csPos[cci][ci
				+ 1][2][0] * (fx1 - cw)
				+ (long) csPos[cci][ci
				+ 1][2][1] * cw)
				>> 16);
		cx = (int) (((long) x1 * (fx1 - co) + (long) x2 * co) >> 16);
		cy = (int) (((long) y1 * (fx1 - co) + (long) y2 * co) >> 16);
		cz = (int) (((long) z1 * (fx1 - co) + (long) z2 * co) >> 16);
		eax = cx;
		if (brk) {
			eay = cy + fx1 / 5;
		} else {
			eay = cy + fx1 / 3;
		}
		eaz = cz;
		sx = cx + ((x2 - x1) >> 3);
		sy = cy + ((y2 - y1) >> 3) + fx1 / 6;
		sz = cz + ((z2 - z1) >> 3);
	}

	/*
	 * Handle title screen operation.
	 */
	// Cursor position.
	private int csp;
	// Button pressed or not.
	private boolean bprs = true;

	private void moveTitle(int key) {
		if (key == 0) {
			bprs = false;
		} else {
			if (bprs)
				return;
			bprs = true;
		}
		if ((key & (1 << Display.KEY_UP)) != 0)
			csp--;
		if ((key & (1 << Display.KEY_DOWN)) != 0)
			csp++;
		csp &= 1;
		if ((key & (1 << Display.KEY_LEFT)) != 0) {
			if (csp == 0) {
				sct--;
			} else {
				sdf--;
			}
		}
		if ((key & (1 << Display.KEY_RIGHT)) != 0) {
			if (csp == 0) {
				sct++;
			} else {
				sdf++;
			}
		}
		if (sct < 0)
			sct = 5;
		if (sct > 5)
			sct = 0;
		if (sdf < 0)
			sdf = 3;
		if (sdf > 3)
			sdf = 0;
	}

	private int cnt;

	/**
	 * Move 1 frame.
	 */
	private void move(int key) {
		if (state == 5)
			return;
		moveViewpoint();
		moveCar(key);
		if (state == 0)
			moveTitle(key);
		for (int i = 0; i < 4; i++) {
			gcl[i][0] += (gacl[i][0] - gcl[i][0]) >> 4;
			gcl[i][1] += (gacl[i][1] - gcl[i][1]) >> 4;
		}
		switch (state) {
			case 2 :
				ltm--;
				ctm += 49;
				if (ltm <= 0) {
					initGameover();
				}
				break;
			case 3 :
			case 4 :
				if (cnt >= 200)
					initTitle();
				break;
		}
		cnt++;
	}

	private boolean prsd = true;

	/**
	 * Check a key pad state.
	 */
	public void handleKeyState(int key) {
		if (key == 0) {
			prsd = false;
		} else {
			if (prsd)
				return;
			prsd = true;
		}
		switch (key) {
			case (1 << Display.KEY_SOFT1) :
				switch (state) {
					case 0 :
						initGame();
						break;
					case 2 :
						initPause();
						break;
					case 5 :
						resumePause();
						break;
				}
				break;
			case (1 << Display.KEY_SOFT2) :
				switch (state) {
					case 0 :
						terminate();
						break;
					default :
						initTitle();
						break;
				}
		}
	}

	private void drawGround(int d3) {
		int yo = (ey - sy) / 2;
		int cr = (gcl[0][0] + gcl[0][1]) / 2,
			cg = (gcl[1][0] + gcl[1][1]) / 2,
			cb = (gcl[2][0] + gcl[2][1]) / 2,
			ca = (gcl[3][0] + gcl[3][1]) / 2;
		g.pushMatrix();
		g.lookAt(0, 0, -fx1, 0, 0, 0, sctbl[d3], sctbl[d3 + 256], 0);
		g.begin(C3DGraphics.C3DL_QUADS);
		gColor(gcl[0][0], gcl[1][0], gcl[2][0], gcl[3][0]);
		g.vertex(-fx1, yo, fx1);
		gColor(cr, cg, cb, ca);
		g.vertex(0, yo, fx1);
		gColor(gcl[0][0] / 2, gcl[1][0] / 2, gcl[2][0] / 2, gcl[3][0]);
		g.vertex(0, yo - (fx1 << 1), fx1);
		g.vertex(-fx1, yo - (fx1 << 1), fx1);
		gColor(cr, cg, cb, ca);
		g.vertex(0, yo, fx1);
		gColor(gcl[0][1], gcl[1][1], gcl[2][1], gcl[3][1]);
		g.vertex(fx1, yo, fx1);
		gColor(gcl[0][1] / 2, gcl[1][1] / 2, gcl[2][1] / 2, gcl[3][1]);
		g.vertex(fx1, yo - (fx1 << 1), fx1);
		g.vertex(0, yo - (fx1 << 1), fx1);
		g.end();
		g.popMatrix();
	}

	private void drawString(int i, int x, int y, int s) {
		g.bindTexture(C3DGraphics.C3DL_TEXTURE_2D, i);
		g.begin(C3DGraphics.C3DL_QUADS);
		g.texCoord(0, fx1 - 3000);
		g.vertex(-x, y, 0);
		g.texCoord(fx1 - 3000, fx1 - 3000);
		g.vertex(-x - stw[i] * s, y, 0);
		g.texCoord(fx1 - 3000, 0);
		g.vertex(-x - stw[i] * s, y + s * 4, 0);
		g.texCoord(0, 0);
		g.vertex(-x, y + s * 4, 0);
		g.end();
	}

	private void drawTime(int t, int x, int y, int s) {
		drawString(t / (60 * 100) % 10 + 1, x, y, s);
		x += fx1 / 32;
		drawString(11, x, y, s);
		x += fx1 / 32;
		drawString(t / (10 * 100) % 6 + 1, x, y, s);
		x += fx1 / 32;
		drawString(t / 100 % 10 + 1, x, y, s);
		x += fx1 / 32;
		drawString(12, x, y, s);
		x += fx1 / 32;
		drawString(t / 10 % 10 + 1, x, y, s);
		x += fx1 / 32;
		drawString(t % 10 + 1, x, y, s);
	}

	/**
	 * Draw to the canvas.
	 */
	private void draw() {
		int d3 = cd3 & 1023;
		int lts;

		g.lock();
		g.clear(C3DGraphics.C3DL_COLOR_BUFFER);
		drawGround(d3);
		g.pushMatrix();
		int aex, aey;
		if (coc) {
			ex = ex + randNS(fx1 / 12);
			ey = ey + randNS(fx1 / 12);
		}
		g.lookAt(ex, ey, ez, sx, sy, sz, sctbl[d3], sctbl[d3 + 256], 0);
		g.pushMatrix();
		g.translate(csEpos[cci][0], csEpos[cci][1], csEpos[cci][2]);
		g.rotate(csEdeg[cci][0], 0, fx1, 0);
		g.rotate(csEdeg[cci][1], fx1, 0, 0);
		g.callList(ncci + 1);
		g.popMatrix();
		g.callList(cci + 1);
		g.popMatrix();

		g.pushMatrix();
		g.lookAt(0, 0, -fx1, 0, 0, 0, 0, fx1, 0);
		g.color(fx1, fx1, fx1, fx1);
		g.enable(C3DGraphics.C3DL_TEXTURE_2D);
		switch (state) {
			case 0 :
				drawString(13, -22000, 22000, fx1 / 80);
				drawString(14, -22000 + fx1 / 32 * 7, 22000, fx1 / 80);
				drawString(19 + sct, 2000, -15000, fx1 / 80);
				drawString(25 + sdf, 2000, -17000, fx1 / 80);
				if ((cnt & 15) < 8) {
					drawString(18, -500, -15000 - csp * 2000, fx1 / 80);
					drawString(17, 22000, -15000 - csp * 2000, fx1 / 80);
				}
				drawString(29, -23000, 15000, fx1 / 80);
				drawTime(rcd[sct][sdf] / 10, -23000, 12000, fx1 / 80);
				g.disable(C3DGraphics.C3DL_TEXTURE_2D);
				g.begin(C3DGraphics.C3DL_LINES);
				g.vertex(30000, 23000, 0);
				g.vertex(-9000, 23000, 0);
				g.end();
				break;
			case 2 :
				lts = ltm / 20;
				if (lts >= 10) {
					drawString(lts / 10 + 1, 21500 - fx1 / 32, 22000, fx1 / 80);
				}
				drawString(lts % 10 + 1, 21500, 22000, fx1 / 80);
				if (ldc + 40 >= cnt && cnt >= ldc) {
					g.color(fx1, fx1 / 3, fx1 / 3, fx1);
					drawString(30, -22000, 22000, fx1 / 80);
					g.color(fx1, fx1, fx1, fx1);
					drawString(lap % 10 + 1, -13500, 22000, fx1 / 80);
					if (tlp < 10) {
						drawString(31, -13500 + fx1 / 32, 22000, fx1 / 80);
						drawString(
							tlp + 1,
							-13500 + fx1 / 32 * 2,
							22000,
							fx1 / 80);
					}
				}
				break;
			case 3 :
				drawString(16, -fx1 / 64 * 8, 2000, fx1 / 80);
				break;
			case 4 :
				drawString(15, -fx1 / 64 * 5, 2000, fx1 / 80);
				drawString(29, -23000, 15000, fx1 / 80);
				drawTime(rcd[sct][sdf] / 10, -23000, 12000, fx1 / 80);
				drawTime(ctm / 10, 9500, -12000, fx1 / 80);
				break;
			case 5 :
				drawString(32, -fx1 / 64 * 5, 2000, fx1 / 80);
				break;
		}
		g.disable(C3DGraphics.C3DL_TEXTURE_2D);
		g.popMatrix();

		g.flush();
		g.unlock(true);
	}

	/**
	 * Main loop. (20 frames/sec)
	 */
	public void run() {
		long prvTicks, nowTicks;
		int frame;

		prvTicks = System.currentTimeMillis();
		for (;;) {
			nowTicks = System.currentTimeMillis();
			frame = (int) (nowTicks - prvTicks) / 50;
			if (frame <= 0) {
				frame = 1;
				prvTicks += 50;
				//while (System.currentTimeMillis() < prvTicks) {}
				try {
					Thread.sleep(prvTicks - System.currentTimeMillis());
				} catch (InterruptedException e) {
				}
			} else {
				prvTicks = nowTicks;
				if (frame > 4) {
					frame = 4;
				}
			}
			if (g == null) {
				g = canvas.getC3DGraphics();
				if (g != null) {
					try {
						initCanvas();
					} catch (Exception e) {
						continue;
					}
				} else {
					continue;
				}
			}
			int key = canvas.getKeypadState();
			handleKeyState(key);
			for (; frame > 0; frame--) {
				move(key);
			}
			draw();
		}
	}

	/*
	 * Random util methods.
	 */
	private static Random rnd = new Random();
	private static Random sdRnd = new Random();

	private int randN(int n) {
		return Math.abs(rnd.nextInt() % n);
	}

	private int randNS(int n) {
		return rnd.nextInt() % n;
	}

	/*
	 * SinCos util methods.
	 */
	private int[] sctbl = new int[1024 + 256];

	private void initSinCos() {
		for (int i = 0; i < 1024 + 256; i++) {
			sctbl[i] = C3DMath.sin(i * (360 * fx1 / 1024));
		}
	}

	/**
	 * Sleep when the phone is folded.
	 */
	public void processSystemEvent(int type, int param) {
		if (type == FOLD_CHANGED_EVENT && param == 0) {
			if (state == 2)
				initPause();
			try {
				sleep();
			} catch (IllegalStateException e) {
				deactivate();
				sleep();
			}
		}
	}

	private final short[] lt =
		{
			0x7bed,
			0x6bee,
			0x7927,
			0x6b6e,
			0x79a7,
			0x79a4,
			0x796f,
			0x5bed,
			0x2492,
			0x126f,
			0x5bad,
			0x4927,
			0x5fed,
			0x5ffd,
			0x7b6f,
			0x7be4,
			0x7b7f,
			0x6bad,
			0x79cf,
			0x7492,
			0x5b6f,
			0x5b6a,
			0x5ffa,
			0x5aad,
			0x5bd2,
			0x72a7,
			0x7b6f,
			0x2492,
			0x73e7,
			0x73cf,
			0x5bc9,
			0x79cf,
			0x79ef,
			0x7249,
			0x7bef,
			0x7bcf,
			0x2400,
			0x5a00,
			0x1511,
			0x4454,
			0x02a0,
			};

	// Width of string textures.
	private int[] stw = new int[33];

	/**
	 * Make a texuture with the text.
	 */
	private void makeStringTexture(int tn, String s) {
		int w, l = s.length();
		if (l == 1) {
			w = 4;
		} else {
			w = 32;
		}
		stw[tn] = w;
		byte[] bp = new byte[w * 8 * 4];
		char c;
		short p, m;
		int k;
		byte rc, gc, bc;
		rc = gc = bc = (byte) 255;
		for (int i = 0, x = 0; i < l; i++, x += 3 /*4*/
			) {
			c = s.charAt(i);
			if (c == ' ') {
				continue;
			} else if (c == '\'') {
				c = (char) ('9' - '0' + 1 + 'z' - 'a' + 1);
			} else if (c == '"') {
				c = (char) ('9' - '0' + 1 + 'z' - 'a' + 1 + 1);
			} else if (c == '<') {
				c = (char) ('9' - '0' + 1 + 'z' - 'a' + 1 + 2);
			} else if (c == '>') {
				c = (char) ('9' - '0' + 1 + 'z' - 'a' + 1 + 3);
			} else if (c == '/') {
				c = (char) ('9' - '0' + 1 + 'z' - 'a' + 1 + 4);
			} else if (c >= 'a' && c <= 'z') {
				c -= 'a';
			} else {
				c = (char) (c - '0' + 'z' - 'a' + 1);
			}
			p = lt[c];
			m = 0x4000;
			for (int j = 0; j < 15; j++) {
				k = ((j % 3) + x + (j / 3) * w) * 4;
				if ((p & m) != 0) {
					bp[k] = rc;
					bp[k + 1] = gc;
					bp[k + 2] = bc;
					bp[k + 3] = (byte) 255;
				} else {
					bp[k] = bp[k + 1] = bp[k + 2] = bp[k + 3] = (byte) 0;
				}
				m >>= 1;
			}
		}
		g.bindTexture(C3DGraphics.C3DL_TEXTURE_2D, tn);
		g.texImage2D(
			C3DGraphics.C3DL_TEXTURE_2D,
			0,
			C3DGraphics.C3DL_RGBA8,
			w,
			8,
			0,
			C3DGraphics.C3DL_RGBA,
			C3DGraphics.C3DL_UNSIGNED_BYTE,
			bp);
	}

	private void initStringTextures() {
		for (int i = 0; i < 10; i++) {
			makeStringTexture(i + 1, new Integer(i).toString());
		}
		makeStringTexture(11, "'");
		makeStringTexture(12, "\"");
		makeStringTexture(13, "urban");
		makeStringTexture(14, "drivin'");
		makeStringTexture(15, "goal");
		makeStringTexture(16, "timeover");
		makeStringTexture(17, "<");
		makeStringTexture(18, ">");
		makeStringTexture(19, "city");
		makeStringTexture(20, "seaside");
		makeStringTexture(21, "mountain");
		makeStringTexture(22, "forest");
		makeStringTexture(23, "lakeside");
		makeStringTexture(24, "suburb");
		makeStringTexture(25, "easy");
		makeStringTexture(26, "normal");
		makeStringTexture(27, "hard");
		makeStringTexture(28, "random");
		makeStringTexture(29, "besttime");
		makeStringTexture(30, "lap");
		makeStringTexture(31, "/");
		makeStringTexture(32, "pause");
	}

	/**
	 * Bootstrap.
	 */
	public void start() {
		initSinCos();
		init();
		run();
	}
}
